/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2021. All rights reserved.
 */

package crypto

import (
	"crypto/sha256"
	"io"
	"os"
	"path/filepath"

	"github.com/pkg/errors"
)

// HashForFile get sha256 hash for file content
func HashForFile(filePath string) ([]byte, error) {
	h := sha256.New()
	file, err := os.Open(filepath.Clean(filePath))
	if err != nil {
		return nil, errors.WithMessage(err, "open file err")
	}
	defer func() {
		if err = file.Close(); err != nil {
			return
		}
	}()
	_, err = io.Copy(h, file)
	if err != nil {
		return nil, errors.WithMessage(err, "io copy err")
	}
	hashInfo := h.Sum(nil)
	return hashInfo, nil
}

// Hash for sha256
func Hash(msg []byte) []byte {
	h := sha256.New()
	_, err := h.Write(msg)
	if err != nil {
		return nil
	}
	hashInfo := h.Sum(nil)
	return hashInfo
}
