/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 */

// Package cryptoimpl is the crypto implementation
package cryptoimpl

import (
	"huawei.com/huaweichain/common/cryptomgr"
)

// SymmetricKeyFactory symmetric key factory
type SymmetricKeyFactory interface {
	Name() string
	NewSymmetricKeyWithRandNum(key []byte) (cryptomgr.SymmetricKey, error)
	NewSymmetricKey() (cryptomgr.SymmetricKey, error)
}

// KeyFactory the key factory.
type KeyFactory interface {
	Name() string
	GenerateKey() (cryptomgr.Key, error)
	GetKeyFromPem(pemKey []byte) (cryptomgr.Key, error)
	GetPubKeyFromDer(derKey []byte) (cryptomgr.Key, error)
	CreateKeyFromBuffer(buffer []byte) (cryptomgr.Key, error)
}

// CertFactory the certificate factory.
type CertFactory interface {
	Name() string
	GenerateSelfSignCert(info *cryptomgr.CertBasicInfo, key cryptomgr.Key) (cryptomgr.Cert, error)
	GenerateMiddleCACert(info *cryptomgr.CertBasicInfo, pubKey cryptomgr.Key, caPriKey cryptomgr.Key,
		caCert cryptomgr.Cert) (cryptomgr.Cert, error)
	GenerateCert(info *cryptomgr.CertBasicInfo, pubKey cryptomgr.Key, caPriKey cryptomgr.Key,
		caCert cryptomgr.Cert) (cryptomgr.Cert, error)
	GetCertFromPem(pemCert []byte) (cryptomgr.Cert, error)
}
