/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2023. All rights reserved.
 */

package cryptoimpl

var certFactoryMap = make(map[string]CertFactory)
var asymmetricKeyFactoryMap = make(map[string]KeyFactory)
var hashFuncMap = make(map[string]func(msg []byte) []byte)
var hashForFileFuncMap = make(map[string]func(string) ([]byte, error))
var symmetricKeyFactoryMap = make(map[string]SymmetricKeyFactory)

// RegisterCertFactory register cert factory
func RegisterCertFactory(f CertFactory) {
	certFactoryMap[f.Name()] = f
}

// GetCertFactory get cert factory
func GetCertFactory(name string) CertFactory {
	return certFactoryMap[name]
}

// RegisterAsKeyFactory register asymmetric key factory
func RegisterAsKeyFactory(f KeyFactory) {
	asymmetricKeyFactoryMap[f.Name()] = f
}

// GetAsKeyFactory get asymmetric key factory
func GetAsKeyFactory(name string) KeyFactory {
	return asymmetricKeyFactoryMap[name]
}

// RegisterHashFunc register hash func map
func RegisterHashFunc(name string, hashFunc func(msg []byte) []byte) {
	hashFuncMap[name] = hashFunc
}

// GetHashFunc get hash func
func GetHashFunc(name string) func([]byte) []byte {
	return hashFuncMap[name]
}

// RegisterHashForFileFunc register hashForFile func map
func RegisterHashForFileFunc(name string, hashForFileFunc func(string) ([]byte, error)) {
	hashForFileFuncMap[name] = hashForFileFunc
}

// GetHashForFileFunc get hashForFile func
func GetHashForFileFunc(name string) func(string) ([]byte, error) {
	return hashForFileFuncMap[name]
}

// RegisterSymmetricKeyFactory register symmetric key factory
func RegisterSymmetricKeyFactory(skf SymmetricKeyFactory) {
	symmetricKeyFactoryMap[skf.Name()] = skf
}

// GetSymmetricKeyFactory get symmetric key factory
func GetSymmetricKeyFactory(name string) SymmetricKeyFactory {
	return symmetricKeyFactoryMap[name]
}
