//go:build !nogm
// +build !nogm

/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 */

package gmcrypto

import (
	"fmt"
	"strings"
	"testing"
	"time"

	"huawei.com/huaweichain/common/cryptomgr"
)

var gmCaCert = `-----BEGIN CERTIFICATE-----
MIICwzCCAmmgAwIBAgIRAJ3DNbiyL/Jj/IRW/d25NAAwCgYIKoEcz1UBg3UwgZMx
DTALBgNVBAYTBHRlc3QxDTALBgNVBAgTBHRlc3QxDTALBgNVBAcTBHRlc3QxDTAL
BgNVBAkTBHRlc3QxDTALBgNVBBETBHRlc3QxGTAXBgNVBAoTEG9yZzEuZXhhbXBs
ZS5jb20xDTALBgNVBAsTBHRlc3QxHDAaBgNVBAMTE2NhLm9yZzEuZXhhbXBsZS5j
b20wHhcNMjAwODE4MTIwNDAwWhcNMzAwODE2MTIwNDAwWjCBkzENMAsGA1UEBhME
dGVzdDENMAsGA1UECBMEdGVzdDENMAsGA1UEBxMEdGVzdDENMAsGA1UECRMEdGVz
dDENMAsGA1UEERMEdGVzdDEZMBcGA1UEChMQb3JnMS5leGFtcGxlLmNvbTENMAsG
A1UECxMEdGVzdDEcMBoGA1UEAxMTY2Eub3JnMS5leGFtcGxlLmNvbTBZMBMGByqG
SM49AgEGCCqBHM9VAYItA0IABEc3yCGBrsaj8Anh+Pway5c6LaZCar10JrsYH7bc
7Qmp1YlDDPj/BV7iWS2vS5wBQ2o0mdW8hAsUVPwTL+hkGiqjgZswgZgwDgYDVR0P
AQH/BAQDAgGmMB0GA1UdJQQWMBQGCCsGAQUFBwMCBggrBgEFBQcDATAPBgNVHRMB
Af8EBTADAQH/MCkGA1UdDgQiBCC7wlVdjmzSGnpWnKahbu64Qu3CJi4ad4teDRNm
UMjQRjArBgNVHSMEJDAigCC7wlVdjmzSGnpWnKahbu64Qu3CJi4ad4teDRNmUMjQ
RjAKBggqgRzPVQGDdQNIADBFAiEAvuntSPrhjio3IhwtVW/RIgzWgM2GgX0Ozw8S
MxzcnoMCICBJNB5LnH2BpAuQQHkGtp3Zo8BW0kvXHjqwQLvPc5cp
-----END CERTIFICATE-----
`

var gmxianwangKey = `-----BEGIN PRIVATE KEY-----
MIGHAgEAMBMGByqGSM49AgEGCCqBHM9VAYItBG0wawIBAQQgf0bbAllYsVEa/ret
qIMu8beGxE5Eg2XdBb+zwECvaKWhRANCAARlvSl/NU1ydfNuazQp3kbx6SyfFwhf
lR92QgT6LtdotA1O2Htcu8wx9RsvBuP8yErrMudL6u2/TnJLmHmVpJOB
-----END PRIVATE KEY-----`
var gmxianwangCert = `-----BEGIN CERTIFICATE-----
MIICnDCCAkOgAwIBAgIRAP/TJRjwvEMHRSNXAb2wiOcwCgYIKoEcz1UBg3UwgZAx
CzAJBgNVBAYTAkNOMRIwEAYDVQQIDAlHdWFuZ2RvbmcxETAPBgNVBAcMCFNoZW56
aGVuMRkwFwYDVQQKDBBjY2NjY2MtN2lsdzIxcmttMQ8wDQYDVQQLDAZIdWF3ZWkx
LjAsBgNVBAMMJWNhLmNjY2NjYy03aWx3MjFya20uc3ZjLmNsdXN0ZXIubG9jYWww
HhcNMjIxMDI0MDMzOTM1WhcNMzIxMDIxMDMzOTM1WjCBkDELMAkGA1UEBhMCQ04x
EjAQBgNVBAgMCUd1YW5nZG9uZzERMA8GA1UEBwwIU2hlbnpoZW4xGTAXBgNVBAoM
EGNjY2NjYy03aWx3MjFya20xDzANBgNVBAsMBkh1YXdlaTEuMCwGA1UEAwwlY2Eu
Y2NjY2NjLTdpbHcyMXJrbS5zdmMuY2x1c3Rlci5sb2NhbDBZMBMGByqGSM49AgEG
CCqBHM9VAYItA0IABGW9KX81TXJ1825rNCneRvHpLJ8XCF+VH3ZCBPou12i0DU7Y
e1y7zDH1Gy8G4/zISusy50vq7b9OckuYeZWkk4GjfDB6MB0GA1UdJQQWMBQGCCsG
AQUFBwMCBggrBgEFBQcDATAdBgNVHQ4EFgQUodBmMsJcdl05tpeAKDZkgzY90YAw
HwYDVR0jBBgwFoAUodBmMsJcdl05tpeAKDZkgzY90YAwCwYDVR0PBAQDAgGmMAwG
A1UdEwQFMAMBAf8wCgYIKoEcz1UBg3UDRwAwRAIgRC9J9dmg5qsnmKnLX8ZZn9uJ
1k1kMZtwIkkytFVSE7UCIEE2QMD9Tv00xD0REIf5cwOpaPrRGc3LHiwIlRWpqEZv
-----END CERTIFICATE-----`

// TestGenerateSelfSignCert test generate self sign cert
func TestGenerateSelfSignCert(t *testing.T) {
	key, err := GeneratePriKey()
	if err != nil {
		t.Fatalf("create gm key failed\n")
		return
	}
	years := 25
	info := &cryptomgr.CertBasicInfo{
		Organization:     "huawei",
		OrganizationUnit: "hanyansuo",
		Country:          "CN",
		Province:         "zhejiang",
		Locality:         "hangzhou",
		CommonName:       "www.test.com",
		ValidationYears:  uint(years),
	}
	cert, err := GenerateSelfSignCert(info, key)
	if err != nil {
		t.Fatalf("create cert failed, err:%s ", err.Error())
	}

	fmt.Printf("the cert content is \n%s\n", string(cert.GetPemCertBytes()))

	numberStr := cert.GetSerialNumber()
	fmt.Printf("the gm serial number is \n%s\n", numberStr)
	if strings.EqualFold(cert.GetCommonName(), "www.test.com") != true {
		t.Fatalf("the cert common name is not correct")
	}
	fmt.Printf("the finger print is \n%s\n", cert.GetFingerPrint())

	if (cert.GetExpireTime().Year()) != (time.Now().Year() + years) {
		t.Fatalf("the time for expire is not %d years, the expire year is %d", years, cert.GetExpireTime().Year())
	}
	fingerPrint := cert.GetFingerPrint()
	t.Logf("the finger print is %s", fingerPrint)

	t.Logf("Generate self sign cert success\n")
}

// TestGenerateCert test generate cert
func TestGenerateCert(t *testing.T) {
	caKey, err := GeneratePriKey()
	if err != nil {
		t.Fatalf("create gm key failed\n")
		return
	}
	years := 17
	info := &cryptomgr.CertBasicInfo{
		Organization:     "huawei",
		OrganizationUnit: "hangyansuo",
		Country:          "CN",
		Province:         "zhejiang",
		Locality:         "hangzhou",
		CommonName:       "www.test.com",
		ValidationYears:  uint(years),
	}
	caCert, err := GenerateSelfSignCert(info, caKey)
	if err != nil {
		t.Fatalf("create cert failed ")
	}

	middlePriKey, err := GeneratePriKey()
	if err != nil {
		t.Fatalf("create myPriKey failed\n")
		return
	}
	middleCertInfo := &cryptomgr.CertBasicInfo{
		Organization:     "huawei",
		OrganizationUnit: "hangyansuo",
		Country:          "CN",
		Province:         "zhejiang",
		Locality:         "hangzhou",
		CommonName:       "middle.www.test.com",
		ValidationYears:  uint(years),
	}
	middleCert, err := GenerateMiddleCACert(middleCertInfo, middlePriKey.GetPublicKey(), caKey, caCert)
	if err != nil {
		t.Fatalf(err.Error())
	}

	if strings.EqualFold(middleCert.GetCommonName(), "middle.www.test.com") != true {
		t.Fatalf("the cert common name is not correct")
	}

	if (middleCert.GetExpireTime().Year()) != (time.Now().Year() + years) {
		t.Fatalf("the time for expire is not %d years", years)
	}

	adminPriKey, err := GeneratePriKey()
	if err != nil {
		t.Fatalf("create myPriKey failed\n")
		return
	}
	endCertInfo := &cryptomgr.CertBasicInfo{
		Organization:     "huawei",
		OrganizationUnit: "hangyansuo",
		Country:          "CN",
		Province:         "zhejiang",
		Locality:         "hangzhou",
		CommonName:       "admin.www.test.com",
		ValidationYears:  uint(years),
	}
	cert, err := GenerateCert(endCertInfo, adminPriKey.GetPublicKey(), middlePriKey, middleCert)
	if err != nil {
		t.Fatalf(err.Error())
	}

	if strings.EqualFold(cert.GetCommonName(), "admin.www.test.com") != true {
		t.Fatalf("the cert common name is not correct")
	}

	if (cert.GetExpireTime().Year()) != (time.Now().Year() + years) {
		t.Fatalf("the time for expire is not %d years", years)
	}
}

// TestGmcert_Verify test gm cert verify
func TestGmcert_Verify(t *testing.T) {
	caKey, err := GeneratePriKey()
	if err != nil {
		t.Fatalf("create gm key failed\n")
		return
	}
	years := 18
	info := &cryptomgr.CertBasicInfo{
		Organization:     "huawei",
		OrganizationUnit: "hangyansuo",
		Country:          "CN",
		Province:         "zhejiang",
		Locality:         "hangzhou",
		CommonName:       "www.test.com",
		ValidationYears:  uint(years),
	}
	caCert, err := GenerateSelfSignCert(info, caKey)
	if err != nil {
		t.Fatalf("create cert failed ")
	}

	msg := []byte("hello")
	sig, err := caKey.Sign(msg, cryptomgr.Sm3)
	if err != nil {
		t.Fatalf("sign failed")
	}

	err = caCert.Verify(nil, sig, cryptomgr.Sm3)
	if err == nil {
		t.Fatalf("should verify failed, but not")
	}
}
