//go:build !nogm
// +build !nogm

/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 */

package gmcrypto

import (
	"strings"

	"github.com/pkg/errors"

	"huawei.com/huaweichain/common/cryptomgr"
	"huawei.com/huaweichain/gmssl/wrapper"
)

const sm3HashFlag = "SM3"

// HashForFile get hash for file content
func HashForFile(filePath string) ([]byte, error) {
	return nil, errors.New("not support to get hash for file content")
}

// Hash for sm3
func Hash(msg []byte) []byte {
	sm3ctx, err := wrapper.NewDigestContext(strings.ToUpper(cryptomgr.Sm3))
	defer sm3ctx.Free()
	if err != nil {
		log.Errorf("New sm3 digest context failed:%s", err.Error())
		return nil
	}
	err = sm3ctx.Reset()
	if err != nil {
		log.Errorf("Sm3 context reset failed:%s", err.Error())
		return nil
	}
	err = sm3ctx.Update(msg)
	if err != nil {
		log.Errorf("Sm3 context update message failed:%s", err.Error())
		return nil
	}
	sm3Hash, err := sm3ctx.Final()
	if err != nil {
		log.Errorf("Sm3 ctx final failed:%s", err.Error())
		return nil
	}
	return sm3Hash
}
