//go:build windows || nogm
// +build windows nogm

/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 */

// Package gmcrypto the gm algorithm for generate of certificate and key
package gmcrypto

import (
	"github.com/pkg/errors"

	"huawei.com/huaweichain/common/cryptomgr"
)

// GmKey gm key
type GmKey struct {
}

// GetPemBytes get pem format key
func (g GmKey) GetPemBytes() []byte {
	return nil
}

// GetDerBytes get der format key
func (g GmKey) GetDerBytes() ([]byte, error) {
	return nil, errors.New("gm is not supported in windows")
}

// IsSymmetric is symmetric key
func (g GmKey) IsSymmetric() bool {
	return false
}

// IsPrivate is private key
func (g GmKey) IsPrivate() bool {
	return false
}

// GetPublicKey get public key object from private key
func (g GmKey) GetPublicKey() cryptomgr.Key {
	return nil
}

// Sign get signature
func (g GmKey) Sign(msg []byte, hashAlg string) ([]byte, error) {
	return nil, errors.New("sign by gm private key is not supported in windows")
}

// Verify verify signature
func (g GmKey) Verify(msg []byte, signature []byte, hashAlg string) error {
	return errors.New("verify by gm public key is not support in windows")
}

// CleanKeySensitiveInfo clean sensitive info in key object
func (g GmKey) CleanKeySensitiveInfo() {
	panic("windows not supported")
}

// GeneratePriKey generate private key
func GeneratePriKey() (cryptomgr.Key, error) {
	return nil, nil
}

// GetPubKey generate public key from pem
func GetPubKey(pemKey []byte) (cryptomgr.Key, error) {
	return nil, nil
}

// GetPriKey generate private key from pem
func GetPriKey(pemKey []byte) (cryptomgr.Key, error) {
	return nil, nil
}

// GetPubKeyFromDer generate public key from der
func GetPubKeyFromDer(derKey []byte) (cryptomgr.Key, error) {
	return nil, nil
}

// CheckGmPriKey check key interface implemented by gm private key struct
func CheckGmPriKey(key cryptomgr.Key) error {
	return nil
}

// CheckGmPubKey check key interface implemented by gm public key struct
func CheckGmPubKey(key cryptomgr.Key) error {
	return nil
}
