/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package tls

import (
	"bufio"
	"bytes"
	"context"
	"fmt"
	"io/ioutil"
	"log"
	"net/http"
	"testing"
)

const rootFile = "../credentials/testdata/root.cert.pem"
const certFile = "../credentials/testdata/CLIENT.cert.pem"
const keyFile = "../credentials/testdata/CLIENT.private.pkcs8.key.pem"

var end chan bool

func ServerRun() {
	cfg := getCfg()
	ln, err := Listen("tcp", ":50052", cfg)
	if err != nil {
		log.Fatal(err)
	}
	defer ln.Close()

	http.HandleFunc("/", func(writer http.ResponseWriter, request *http.Request) {
		b, err := ioutil.ReadAll(request.Body)
		if err != nil {
			log.Fatal(err)
		}
		fmt.Fprintf(writer, "accept: %s", b)
	})
	err = http.Serve(ln, nil)
	if err != nil {
		log.Fatal(err)
	}
}

func gmClientRun() {
	cfg := getCfg()
	conn, err := Dial("tcp", "localhost:50052", cfg)
	if err != nil {
		log.Fatal(err)
	}
	defer conn.Close()

	ctx := context.Background()
	request, err := http.NewRequestWithContext(ctx, http.MethodPost, "http://localhost:50052/aa", bytes.NewBuffer([]byte("aaa")))
	if err != nil {
		log.Fatal(err)
	}
	if err = request.Write(conn); err != nil {
		log.Fatal(err)
	}

	reader := bufio.NewReader(conn)
	_, err = reader.Peek(1)
	if err != nil {
		log.Fatal(err)
	}
	res, err := http.ReadResponse(reader, request)
	if err != nil {
		log.Fatal(err)
	}
	b, err := ioutil.ReadAll(res.Body)
	if err != nil {
		log.Fatal(err)
	}
	if string(b) != "accept: aaa" {
		log.Fatalf("expect 'accept: aaa', real '%v'", string(b))
	}
	end <- true
}

func getCfg() *Config {
	key, err := ioutil.ReadFile(keyFile)
	if err != nil {
		log.Fatal(err)
	}
	cert, err := ioutil.ReadFile(certFile)
	if err != nil {
		log.Fatal(err)
	}
	root, err := ioutil.ReadFile(rootFile)
	if err != nil {
		log.Fatal(err)
	}

	return &Config{
		Key:        key,
		Certs:      [][]byte{cert},
		Roots:      [][]byte{root},
		ServerName: "oss.PMS.com",
	}
}

func Test_tls(t *testing.T) {
	end = make(chan bool, 64)
	go ServerRun()
	go gmClientRun()
	<-end
}
