/* *********************************************************************************************************************
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 * Description: GM
 * Create     : 2022-02-21
 * ****************************************************************************************************************** */

#ifndef GM_CERTIFICATE_H
#define GM_CERTIFICATE_H

// the included header files
#include <stdlib.h>
#include <openssl/bio.h>
#include <openssl/evp.h>
#include <openssl/pem.h>
#include <openssl/x509v3.h>
#include <openssl/err.h>

#define BN_RAND_BITS_LENGTH 128
#define CERT_INFO_LENGTH 512
#define GM_TIME_ADJ_YEAR_ATTRIBUTE 31536000

typedef struct {
    unsigned char *country;
    unsigned char *province;
    unsigned char *locality;
    unsigned char *org;
    unsigned char *ou;
    unsigned char *commonName;
    uint years;
} CertInfo;

long BIO_get_mem_data_func(BIO *b, char **pp);
void OPENSSL_free_func(void *addr);
int GetFingerPrintf(const X509 *x, char *fingerPrintBuf, const uint bufLen);
int VerifyCert(X509 *cert, X509 *caCert);
X509 *GenerateCertBasic(CertInfo *certInfo, EVP_PKEY *pubKey);
X509 *GenerateMiddleCACert(CertInfo *certInfo, EVP_PKEY *pubKey, EVP_PKEY *caPriKey, X509 *caCert);
X509 *GenerateCert(CertInfo *certInfo, EVP_PKEY *pubKey, EVP_PKEY *caPriKey, X509 *caCert);
X509 *GenerateSelfSignCert(CertInfo *certInfo, EVP_PKEY *pkey);
void X509ToPEM(X509 *cert, char *pemBuf, uint32_t *pemBufLen);
int GetExipreTime(X509 *x, char *timeBuf, uint bufLen);
CertInfo *NewCertInfo(void);
_Bool SetCountryProvinceLocality(CertInfo *certInfo, const char *country, const char *province, const char *locality);
_Bool SetOrgOuCommonNameYears(CertInfo *certInfo, const char *org, const char *ou, const char *commonName,
    const uint32_t years);
void CertInfoFree(CertInfo *certInfo);
int GetSignAlgorithm(X509 *x, char *sigAlgBuf, uint *bufLen);
int isCaCert(X509* certX509);
#endif
