/* *********************************************************************************************************************
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 * Description: GM
 * Create     : 2022-02-21
 * ****************************************************************************************************************** */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <openssl/ec.h>
#include <openssl/sm2.h>
#include <openssl/bio.h>
#include <openssl/evp.h>
#include <openssl/pem.h>
#include <openssl/err.h>
#include <openssl/engine.h>
#include <openssl/objects.h>
#include <openssl/opensslconf.h>

char *g_ID = "1234567812345678";
int g_IDLen = 16;

EVP_PKEY_CTX *NewPkeyKeygenCtx(const char *alg, ENGINE *e) {
	EVP_PKEY_CTX *ret = NULL;
	EVP_PKEY_CTX *ctx = NULL;
	const EVP_PKEY_ASN1_METHOD *ameth;
	ENGINE *eng = NULL;
	int pkey_id;

	if (!(ameth = EVP_PKEY_asn1_find_str(&eng, alg, -1))) {
		return NULL;
	}
	EVP_PKEY_asn1_get0_info(&pkey_id, NULL, NULL, NULL, NULL, ameth);
	ENGINE_finish(eng);
	if (!(ctx = EVP_PKEY_CTX_new_id(pkey_id, e))) {
		goto end;
	}
	ret = ctx;
	ctx = NULL;
end:
	EVP_PKEY_CTX_free(ctx);
	return ret;
}

EVP_PKEY *PemReadBioPubKey(BIO *bio) {
	return PEM_read_bio_PUBKEY(bio, NULL, NULL, NULL);
}

int PemWriteBioPubKey(BIO *bio, EVP_PKEY *pkey) {
	return PEM_write_bio_PUBKEY(bio, pkey);
}

unsigned char *SkSign(EVP_PKEY *sk, const unsigned char *digest,
	size_t digestLen, size_t *sigLen, ENGINE *e) {
	unsigned char *ret = NULL;
	EVP_PKEY_CTX *ctx = NULL;
	unsigned char *sig = NULL;

	if (!(ctx = EVP_PKEY_CTX_new(sk, e))) {
		return NULL;
	}
	if (EVP_PKEY_sign_init(ctx) <= 0) {
		goto end;
	}
	if (EVP_PKEY_id(sk) == EVP_PKEY_EC && EC_GROUP_get_curve_name(
		EC_KEY_get0_group(EVP_PKEY_get0_EC_KEY(sk))) == NID_sm2p256v1) {
		if (EVP_PKEY_CTX_set_ec_scheme(ctx, NID_sm_scheme) <= 0) {
			goto end;
		}
	}
	if (EVP_PKEY_size(sk) <= 0) {
		goto end;
	}
	if (!(sig = OPENSSL_zalloc(EVP_PKEY_size(sk)))) {
		goto end;
	}
	*sigLen = EVP_PKEY_size(sk);
	if (EVP_PKEY_sign(ctx, sig, sigLen, digest, digestLen) <= 0) {
		goto end;
	}
	ret = sig;
	sig = NULL;
end:
	EVP_PKEY_CTX_free(ctx);
	OPENSSL_free(sig);
	return ret;
}

int PkVerify(EVP_PKEY *pk, EVP_PKEY_CTX *ctx, const unsigned char *digest,
    	size_t digestLen, const unsigned char *sig, size_t sigLen, ENGINE *e) {
	int ret = -1;

	if ((ret = EVP_PKEY_verify(ctx, sig, sigLen, digest, digestLen)) <= 0) {
		printf("ret = %d\n", ret);
		ERR_print_errors_fp(stderr);
		goto end;
	}
end:
	return ret;
}

EVP_PKEY_CTX* GetPubKeyCtxFromPk(EVP_PKEY *pk) {
	EVP_PKEY_CTX *ctx = NULL;
	if (!(ctx = EVP_PKEY_CTX_new(pk, NULL))) {
		printf("%s %d: error\n", __FILE__, __LINE__);
		return NULL;
	}
	if (!EVP_PKEY_verify_init(ctx)) {
		printf("%s %d: error\n", __FILE__, __LINE__);
		return NULL;
	}
    if (EVP_PKEY_CTX_set_ec_scheme(ctx, NID_sm_scheme) <= 0) {
        return NULL;
    }

	return ctx;
}

int ComputeIDDigest(EVP_PKEY *pkey, unsigned char *outBuf, unsigned long *bufLen) {
    return SM2_compute_id_digest(EVP_sm3(), g_ID, g_IDLen, outBuf, (size_t*)&bufLen, EVP_PKEY_get0_EC_KEY(pkey));
}


const EVP_MD * GetSm3DigestName() {
    const char *sm3 = "SM3";
    return EVP_get_digestbyname(sm3);
}




