/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package event

import "sync/atomic"

// Service define the event service interface.
type Service interface {
	TryHold() bool
	Close()
}

func tryHold(holdCount *int32) bool {
	update := func(prev int32) int32 {
		if prev > 0 {
			return prev + 1
		}
		return prev
	}
	count := updateAndGet(holdCount, update)
	return count > 0
}

func close(holdCount *int32) int32 {
	update := func(prev int32) int32 {
		if prev > 0 {
			return prev - 1
		}
		return -1
	}
	return updateAndGet(holdCount, update)
}

func updateAndGet(x *int32, updateFunc func(int32) int32) int32 {
	prev := atomic.LoadInt32(x)
	next := updateFunc(prev)
	for !atomic.CompareAndSwapInt32(x, prev, next) {
		prev = atomic.LoadInt32(x)
		next = updateFunc(prev)
	}
	return next
}
