/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2022-2022. All rights reserved.
 */

package event

import (
	"sync"

	"github.com/pkg/errors"
)

// Mgr is the definition of service manager.
type Mgr struct {
	m sync.Map
}

// Get is used to get the service of the specified key.
func (m *Mgr) Get(key string, supply func() (Service, error)) (Service, error) {
	value, ok := m.m.Load(key)
	if ok {
		service, ok := value.(Service)
		if !ok {
			return nil, errors.New("type assert error: Service")
		}
		if service.TryHold() {
			return service, nil
		}
	}

	service, err := supply()
	if err != nil {
		return nil, errors.WithMessage(err, "supply service error")
	}
	for {
		value, loaded := m.m.LoadOrStore(key, service)
		if !loaded {
			log.Debug("store new service success")
			return service, nil
		}
		oldService, ok := value.(Service)
		if !ok {
			return nil, errors.New("type assert error: Service")
		}
		if oldService.TryHold() {
			return oldService, nil
		}
	}
}

// Remove is used to remove the service of the specified key.
func (m *Mgr) Remove(key string) {
	m.m.Delete(key)
}
