/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2020. All rights reserved.
 */

package config

import (
	"os"
	"reflect"
	"testing"
)

func Test_Client_Config(t *testing.T) {
	configFile := "testdata/client-conf.yml"
	_, errMsg := os.Stat(configFile)
	if errMsg != nil {
		t.Errorf("CfgFilePath %s does not exist", configFile)
		return
	}

	cfg, err := NewClientConfig(configFile)
	if err != nil {
		t.Errorf("new genesis config error: %v\n", err)
		return
	}

	t.Logf("client: %v\n", cfg.Client)
	for i, node := range cfg.Nodes {
		t.Logf("node %v:  %v", i, node)
		t.Logf("node %s request port = %d, rest port %d", node.ID, node.Port, node.RestPort)
		if node.ID == "node1_org1" {
			if reflect.DeepEqual(node, testConf.Nodes[0]) {
				t.Errorf("read node config %+v doesn't eqaul default config %+v", node, testConf.Nodes[0])
			}
			t.Logf("%s compare success", node.ID)
		}
	}
}

var testConf = ClientConfig{
	Client: &Client{
		Type: "",
		Identity: Identity{
			CertPath: "",
			KeyPath:  "",
		},
		TLS: TLS{
			Enable:   false,
			CertPath: "",
			KeyPath:  "",
			RootPath: nil,
		},
	},
	Nodes: []*Node{
		{
			ID:           "node1_org1",
			HostOverride: "peer0.org1.example.com",
			Host:         "peer0.org1.example.com",
			Port:         12020,
			RestPort:     611,
			RPCCompress:  true,
		},
	},
}
