/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

package config

import (
	"regexp"

	"github.com/pkg/errors"

	"huawei.com/huaweichain/proto/common"
)

// CheckEntryPoint returns error when entrypoint information has invalid field.
func CheckEntryPoint(entrypoint *common.Entrypoint) error {
	if err := verifyHostPort(entrypoint.Linkers); err != nil {
		return err
	}
	if err := verifyHostPort(entrypoint.Seeds); err != nil {
		return err
	}
	if err := CheckZoneID(entrypoint.ZoneId); err != nil {
		return err
	}
	if err := verifyEntry(entrypoint); err != nil {
		return err
	}
	return nil
}

const hostPortReg = `^(.*)(:)([0-9]+)$`

func verifyHostPort(s []string) error {
	r, err := regexp.Compile(hostPortReg)
	if err != nil {
		return errors.WithMessagef(err, "regexp compile error: %v", hostPortReg)
	}
	for i := 0; i < len(s); i++ {
		if !r.Match([]byte(s[i])) {
			return errors.Errorf("port error: %v", s[i])
		}
	}
	return nil
}

func verifyEntry(e *common.Entrypoint) error {
	if e.InitialMaster {
		// sub zone master should have linkers
		if e.ZoneId != rootZone && len(e.Linkers) == 0 {
			return errors.Errorf("node is Master of subZone[%v], should have at least one linker", e.ZoneId)
		}
		return nil
	}
	// peer should have seeds
	if len(e.Seeds) == 0 {
		return errors.Errorf("peer should have at least one seed")
	}
	return nil
}
