//go:build gm && linux
// +build gm,linux

package crypto

import (
	"google.golang.org/grpc/credentials"

	"huawei.com/huaweichain/common/cryptomgr"
	gm "huawei.com/huaweichain/gmssl/credentials"
	_ "huawei.com/huaweichain/gmssl/gmcrypto"
	"huawei.com/huaweichain/gmssl/tls"
)

// nolint:gochecknoinits // register GM crypto for tls configuration.
func init() {
	registry[cryptomgr.Sm2WithSm3] = func(serverName string, t *TLS) (credentials.TransportCredentials, error) {
		return gm.NewTLS(&tls.Config{
			ServerName: serverName,
			Certs:      [][]byte{t.certPEMBlock},
			Key:        t.keyPEMBlock,
			Roots:      t.roots,
		}), nil
	}
}

// GmTLSClientConf creates tls configuration for GM.
func GmTLSClientConf(host string, t *TLS) *tls.Config {
	return &tls.Config{
		ServerName: host,
		Key:        t.keyPEMBlock,
		Certs:      [][]byte{t.certPEMBlock},
		Roots:      t.roots,
	}
}
