/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

package flow

import (
	"encoding/json"
	"testing"
	"time"

	"huawei.com/huaweichain/common/logger"
	"huawei.com/huaweichain/sdk/client"
)

func TestAddConsenter(t *testing.T) {
	logger.InitTestingLogger("debug")
	lg := logger.TestingLogger()
	sdk := client.NewEmptyGatewayClient()
	c1 := Consenter{
		Name:     "node2_org1",
		Org:      "org1",
		Addr:     "peer1.org1.example.com",
		RpcPort:  22010,
		ReqPort:  21010,
		RestPort: 20010,
		PemCert:  []byte{1},
	}
	c2 := Consenter{
		Name:     "node1_org2",
		Org:      "org2",
		Addr:     "peer0.org2.example.com",
		RpcPort:  22010,
		ReqPort:  21010,
		RestPort: 20010,
		PemCert:  []byte{2},
	}
	c3 := Consenter{
		Name:     "node2_org2",
		Org:      "org2",
		Addr:     "peer1.org2.example.com",
		RpcPort:  22010,
		ReqPort:  21010,
		RestPort: 20010,
		PemCert:  []byte{3},
	}

	result, err := AddConsenter(sdk, "chain_id", "node1_org1", []string{"peer0.org1.example.com:8080"}, []Consenter{c1, c2, c3}...)
	if err != nil {
		t.Fatal(err)
	}
	for {
		select {
		case res, ok := <-result.Response():
			if !ok {
				if err := result.Err(); err != nil {
					lg.Errorf("%+v", err)
				} else {
					return
				}
			}
			switch res.StepCode() {
			case Abort:
				for n, err := range res.NodeErrs {
					lg.Errorf("node %s occurs error %+v", n, err)
				}
			case Success:
			case Synchronized:
			case Pending:
			case Completed:
			default:
				jsonBytes, err := json.MarshalIndent(res, "", "	")
				if err != nil {
					return
				}
				lg.Debugf("%s", jsonBytes)
			}
		case <-time.After(60 * time.Second):
			return
		}
	}
}

func TestRemoveConsenter(t *testing.T) {
	logger.InitTestingLogger("debug")
	lg := logger.TestingLogger()
	sdk := client.NewEmptyGatewayClient()
	c1 := Consenter{
		Name:     "node1_org2",
		Org:      "org2",
		Addr:     "peer0.org2.example.com",
		RpcPort:  22010,
		ReqPort:  21010,
		RestPort: 20010,
		PemCert:  []byte{2},
	}

	result, _ := RemoveConsenter(sdk, "chain_id", "node1_org1", []Consenter{c1}...) // 校验列表
	for {
		select {
		case _ = <-result.Response():
			lg.Info("test")
		}
	}
}
