/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

// Package flow is the implementation of flow sdk for go client.
package flow

import (
	"huawei.com/huaweichain/common/logger"
	"huawei.com/huaweichain/sdk/client"
	"huawei.com/huaweichain/sdk/flow/internal"
)

// Flow is the root struct
type Flow struct {
	stub internal.Stub
}

var log = logger.GetModuleLogger("sdk.flow", "default")

// MakeFlow is used to create a empty flow instance
func MakeFlow() *Flow {
	return &Flow{
		stub: internal.NewStub(),
	}
}

// LoadFlow creates a flow instance from manual client sdk.
func LoadFlow(client *client.GatewayClient) *Flow {
	return &Flow{
		stub: internal.LoadStub(client),
	}
}

func loadStub(stub internal.Stub) *Flow {
	return &Flow{
		stub: stub,
	}
}

// Identity is used to return a IdentityFlow instance
func (f *Flow) Identity() *IdentityFlow {
	return newIdentityFlow(f)
}

// TLS is used to return a TlSFlow instance
func (f *Flow) TLS() *TlSFlow {
	return newTLSFlow(f)
}

// Router is used to return a RouterFlow instance
func (f *Flow) Router() *RouterFlow {
	return newRouterFlow(f)
}

// Admin is used to return a AdminFlow instance
func (f *Flow) Admin() *AdminFlow {
	return newAdminFlow(f.stub)
}

// Operator is used to return a OperatorFlow instance
func (f *Flow) Operator() *OperatorFlow {
	return newOperatorFlow(f.stub)
}

// User is used to return a UserFlow instance
func (f *Flow) User() *UserFlow {
	return newUserFlow(f.stub)
}

// Check returns error from basic setting for flow instance.
func (f *Flow) Check() error {
	err := f.stub.Err()
	if err != nil {
		return err
	}
	return nil
}

// Destroy clean sensitive information for the flow client, then the flow instance can't be used anymore.
func (f *Flow) Destroy() {
	f.stub.Destroy()
}
