/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

package flow

import (
	"huawei.com/huaweichain/common/cryptomgr"
)

// IdentityFlow is used to set identity config
type IdentityFlow struct {
	f       *Flow
	decrypt func([]byte) []byte
}

func newIdentityFlow(f *Flow) *IdentityFlow {
	return &IdentityFlow{
		f: f,
		decrypt: func(bytes []byte) []byte {
			return bytes
		},
	}
}

// KeyPair is used to set identity client config
func (i *IdentityFlow) KeyPair(alg CryptoSuite, key []byte, cert []byte) *IdentityFlow {
	i.f.stub.SetIdentity(string(alg), i.decrypt(key), i.decrypt(cert))
	return i
}

// Decrypt is used to set decrypt method
func (i *IdentityFlow) Decrypt(decrypt func([]byte) []byte) *IdentityFlow {
	i.decrypt = decrypt
	return i
}

// Then returns basic Flow instance.
func (i *IdentityFlow) Then() *Flow {
	return i.f
}

// CryptoSuite represents identity used crypto algorithm.
type CryptoSuite string

// Optional CryptoSuite contains ecdsa_with_sha236 and sm2_with_sm3.
const (
	CryptoEcdsa CryptoSuite = cryptomgr.EcdsaWithSha256
	CryptoGm    CryptoSuite = cryptomgr.Sm2WithSm3
)
