/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

// Package cgo is wrapper of ffi.
package cgo

/*
#cgo CFLAGS: -I${SRCDIR}/../packaged/include
#cgo LDFLAGS: -lxmart_codec -ldl -lpthread -lm

#include <stdlib.h>
#include "xmart_codec.h"
*/
import "C"
import (
	"reflect"
	"unsafe"

	"github.com/pkg/errors"

	"huawei.com/huaweichain/proto"
	"huawei.com/huaweichain/proto/xmart"
	_ "huawei.com/huaweichain/sdk/flow/internal/packaged/include"
	_ "huawei.com/huaweichain/sdk/flow/internal/packaged/lib"
)

// EncodeXmartInput return scale encoded input data.
func EncodeXmartInput(args *xmart.XmartContractArgs) ([]byte, error) {
	argsBytes, err := proto.Marshal(args)
	if err != nil {
		return nil, errors.WithMessage(err, "failed to marshal XmartContractArgs")
	}
	ctx := newContext()
	res := C.encodeXmartInput(unsafe.Pointer(ctx),
		(*C.uchar)(unsafe.Pointer((*reflect.SliceHeader)(unsafe.Pointer(&argsBytes)).Data)),
		C.size_t(len(argsBytes)))
	if res != 0 {
		return nil, errors.New("encode input in xmart_codec failed")
	}

	return ctx.res, nil
}

// DecodeXmartRes return decoded result.
func DecodeXmartRes(metadata []byte, eRes []byte, method string) string {
	methodStr := C.CString(method)
	defer C.free(unsafe.Pointer(methodStr))
	res := C.decodeRes(
		(*C.uchar)(unsafe.Pointer((*reflect.SliceHeader)(unsafe.Pointer(&metadata)).Data)),
		C.size_t(len(metadata)),
		(*C.uchar)(unsafe.Pointer((*reflect.SliceHeader)(unsafe.Pointer(&eRes)).Data)),
		C.size_t(len(eRes)),
		methodStr)

	return C.GoString(res)
}

type context struct {
	res []byte
}

func newContext() *context {
	return &context{res: nil}
}

func (c *context) setResult(out []byte) {
	c.res = out
}

//export set_rs_result
func set_rs_result(ptr unsafe.Pointer, out *C.uchar, len C.size_t) {
	ctx := (*context)(ptr)
	outBytes := C.GoBytes(unsafe.Pointer(out), C.int(len))
	ctx.setResult(outBytes)
}
