/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

package internal

import (
	"github.com/pkg/errors"

	"huawei.com/huaweichain/proto"
	"huawei.com/huaweichain/proto/common"
)

// HandleResult unwraps the RawMessage, if success returns Response, otherwise returns error.
func HandleResult(resMsg *common.RawMessage) (*common.Response, error) {
	var res common.Response
	if err := proto.Unmarshal(resMsg.Payload, &res); err != nil {
		return nil, errors.WithMessage(err, "failed to unmarshal response")
	}
	if res.Status != common.SUCCESS {
		return nil, errors.Errorf("failed to invoke rpc with error code: %s and msg: %s",
			res.Status, res.StatusInfo)
	}
	return &res, nil
}
