/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

package flow

import (
	"context"

	"huawei.com/huaweichain/proto/nodeservice"
	"huawei.com/huaweichain/sdk/flow/internal"
)

// OperatorFlow is the role of operator
// which is to invoke config/lifecycle flow streamly
type OperatorFlow struct {
	stub internal.Stub
}

func newOperatorFlow(stub internal.Stub) *OperatorFlow {
	return &OperatorFlow{
		stub: stub,
	}
}

// Config is used to return ConfigFlow
func (o *OperatorFlow) Config(chain string) *ConfigFlow {
	return newConfigFlow(chain, o.stub)
}

// Lifecycle is used to return LifecycleFlow
func (o *OperatorFlow) Lifecycle(chain string) *LifecycleFlow {
	return newLifecycleFlow(chain, o.stub)
}

// Archive is used to return ArchiveFlow
func (o *OperatorFlow) Archive(chain, node string) *ArchiveFlow {
	return newArchiveFlow(o.stub, chain, node)
}

func (o *OperatorFlow) Ops(chain, node string) *OpsFlow {
	return &OpsFlow{chain: chain, node: node, stub: o.stub}
}

type OpsFlow struct {
	chain string
	node  string
	stub  internal.Stub
}

func (o *OpsFlow) RemoveFaultyNode(ctx context.Context, host string) error {
	client, err := o.client()
	if err != nil {
		return err
	}
	req := nodeservice.OpsRequest{
		Chain:   o.chain,
		Type:    nodeservice.NETWORK,
		Ops:     "remove_faulty_node",
		Payload: []byte(host),
	}
	msg, err := o.stub.Seal(&req)
	if err != nil {
		return err
	}
	rawRes, err := client.Operate(ctx, msg)
	if err != nil {
		return err
	}
	_, err = internal.HandleResult(rawRes)
	if err != nil {
		return err
	}
	return nil
}

func (o *OpsFlow) client() (nodeservice.OpsClient, error) {
	n, err := o.stub.GetNode(o.node)
	if err != nil {
		return nil, err
	}
	conn, err := n.ChainAction.GetReadyConn()
	if err != nil {
		return nil, err
	}
	return nodeservice.NewOpsClient(conn), nil
}
