/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

package flow

import (
	"github.com/pkg/errors"

	"huawei.com/huaweichain/sdk/flow/types"
)

// RouterFlow is used to set node
type RouterFlow struct {
	f *Flow
}

func newRouterFlow(f *Flow) *RouterFlow {
	return &RouterFlow{
		f: f,
	}
}

// AddNode is used to add nodes for sdk
func (n *RouterFlow) AddNode(nodes ...*types.Node) *RouterFlow {
	for _, node := range nodes {
		n.f.stub.AddNode(node)
	}
	return n
}

// RemoveNode removes remote node by node id
// which delete node instance from router table and close all related connections.
func (n *RouterFlow) RemoveNode(nodes ...string) error {
	for _, node := range nodes {
		if err := n.f.stub.RemoveNode(node); err != nil {
			return errors.WithMessagef(err, "failed to remove node %s from router table", node)
		}
	}
	return nil
}

// Node is used to get the struct Node
func (n *RouterFlow) Node(name string) (*types.Node, error) {
	return n.f.stub.Node(name)
}

// Then returns basic Flow instance.
func (n *RouterFlow) Then() *Flow {
	return n.f
}
