/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

package flow

// TlSFlow is used to set tls config
type TlSFlow struct {
	f       *Flow
	decrypt func([]byte) []byte
	roots   [][]byte
}

func newTLSFlow(f *Flow) *TlSFlow {
	tlsFlowInstance := TlSFlow{
		f: f,
		decrypt: func(bytes []byte) []byte {
			return bytes
		},
	}
	return &tlsFlowInstance
}

// Roots is used to set ca certs for sdk
func (t *TlSFlow) Roots(roots [][]byte) *TlSFlow {
	t.roots = append([][]byte{}, roots...)
	return t
}

// Decrypt is used to set decrypt for sdk
func (t *TlSFlow) Decrypt(decrypt func([]byte) []byte) *TlSFlow {
	t.decrypt = decrypt
	return t
}

// KeyPair is used to set keys and certs for sdk
func (t *TlSFlow) KeyPair(key []byte, cert []byte) *TlSFlow {
	caCerts := make([][]byte, 0, len(t.roots))
	for i := 0; i < len(t.roots); i++ {
		caCerts = append(caCerts, t.decrypt(t.roots[i]))
	}
	t.f.stub.SetTLS(t.decrypt(cert), t.decrypt(key), caCerts)
	return t
}

// Then returns basic Flow instance.
func (t *TlSFlow) Then() *Flow {
	return t.f
}
