/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2023-2023. All rights reserved.
 */

// Package flow is the implementation of flow sdk for go client.
package flow

import (
	"huawei.com/huaweichain/sdk/flow/event"
	"huawei.com/huaweichain/sdk/flow/internal"
	"huawei.com/huaweichain/sdk/flow/types"
)

// UserFlow is the role of user
// which is to streaming invoke contract flow
type UserFlow struct {
	*Flow
}

func newUserFlow(stub internal.Stub) *UserFlow {
	UserFlowInstance := UserFlow{
		Flow: &Flow{stub: stub},
	}
	return &UserFlowInstance
}

// Contract is used to return ContractFlow
func (u *UserFlow) Contract(chain, contract string) *ContractFlow {
	return newContractFlow(u.stub, chain, contract)
}

// EVMContract returns contract flow for specific evm platform.
func (u *UserFlow) EVMContract(chain string) *EVMContractFlow {
	return newEVMContractFlow(u.Contract(chain, types.NativeContractEVM))
}

// Event returns event flow for event querying and listening.
func (u *UserFlow) Event() *event.Flow {
	return event.NewEventFlow(u.stub)
}

// Observer is used to return a ObserverFlow instance
func (u *UserFlow) Observer() *ObserverFlow {
	return newObserverFlow(u.stub)
}
