/*
 *  Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

package rawmessage

import (
	"sort"
	"testing"

	"huawei.com/huaweichain/proto/common"
	"huawei.com/huaweichain/sdk/config"
)

func TestCheckAddZoneCfg(t *testing.T) {
	z := &common.ZoneConf{
		BlockBatch: 5,
		Attachable: true,
		Alloc: &common.Allocator{
			Type:              0,
			MaxPeerNum:        5,
			MaxCoordinatorNum: 5,
			Alloc:             nil,
		},
	}
	domains := []string{"/domainA", "/domainB"}

	zones := []*common.ZoneProperties{
		{
			Id: "zoneB",
			Properties: []*common.Property{
				{
					K: config.AllocCoordinatorNumKey,
					V: &common.Property_I32{I32: int32(z.Alloc.MaxCoordinatorNum)},
				},
				{
					K: config.AllocPeerNumKey,
					V: &common.Property_I32{I32: int32(z.Alloc.MaxPeerNum)},
				},
				{
					K: config.AllocTypeKey,
					V: &common.Property_Str{Str: "balance"},
				},
				{
					K: config.BlockBatchKey,
					V: &common.Property_I32{I32: int32(z.BlockBatch)},
				},
				{
					K: config.DomainKey,
					V: &common.Property_StrArr_{StrArr: &common.Property_StrArr{Ss: domains}},
				},
			},
		},
	}

	sort.Slice(zones, func(i, j int) bool {
		return zones[i].Id < zones[j].Id
	})
	add := &common.ZoneOp_Add{Zones: zones}
	op := &common.ZoneOp{Op: &common.ZoneOp_Add_{Add: add}}
	err := checkAddCfg(op.GetAdd())

	if err != nil {
		t.Errorf("%+v", err)
	}
}
