/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2021-2021. All rights reserved.
 */

package version

import (
	"strconv"
	"strings"

	"github.com/pkg/errors"
)

const (
	segCount = 2
)

// ValidatePlatformVersion check if the given version string is a valid
// version string.
func ValidatePlatformVersion(ver string) error {
	if ver == "" {
		return errors.New("new min platform version of chain should be specified")
	}
	segments := strings.Split(ver, sep)
	if len(segments) != segCount {
		return errors.Errorf(
			"min platform version should contain 2 segments, but got %d", len(segments))
	}
	for _, seg := range segments {
		_, err := strconv.ParseUint(seg, base, bitSize)
		if err != nil {
			return errors.Errorf("segment %s is not numeric: %s", seg, err)
		}
	}
	return nil
}
